<?
namespace Avos\Redis;

//
// return sites redis prefix optionally with /key appended
//

function getPrefix($key='')
{
	global $_AVOS;

	if (!isset($_AVOS['config']['redis']['prefix'])) {
		return $key;
	}

	if (!empty($key)) {
		$key = '/' . $key;
	}

	return $_AVOS['config']['redis']['prefix'] . $key;
}

function handle()
{
	global $_AVOS;
	return $_AVOS['redis'];
}

function set($key, $k, $v=false, $expiry=false)
{
    $redis = handle();

    switch ($key[0]) {
        case ':':
            $key = getSitesPrefix(ltrim($key, ':'));
            break;

        case '^':
            $key = ltrim($key, '^');
            break;

        default:
            $key = getPrefix($key);
    }

    if ($v != false) {
        // set a hash element
        $redis->hSet($key, $k, $v);
    }
    else {
        // set the value
        $redis->set($key, $k);
    }

    if ($expiry) {
        $redis->expire($key, $expiry);
    }
}

function type($key)
{
    $redis = handle();

    if ($key[0] == ':') {
        $key = getSitesPrefix(ltrim($key, ':'));
    }
    else {
        $key = getPrefix($key);
    }

    return $redis->type($key);
}

function getSitesPrefix($key='')
{
    $t = [];
    foreach (explode(',', getConfig('webcams', 'sitesFilter', '')) as $s) {
        $t[$s] = 1;
    }
    ksort($t);
    $prefix = 's/' . implode('', array_keys($t));

	if (!empty($key)) {
		$key = '/' . $key;
    }

    return $prefix . $key;
}

//
// if $key begins with colon like ":models" then instead of prefixing the key with the redis prefix
// as in "mysite/models" use a concat of the sitesFilter before the key as in "cbsclj/models" so data 
// specific to that list of webcam sponsors can be shared between multiple avos sites
//

function get($key, $default=false, $hkey=false)
{
    $redis = handle();

    switch ($key[0]) {
        case ':':
            $key = getSitesPrefix(ltrim($key, ':'));
            break;

        case '^':
            $key = ltrim($key, '^');
            break;

        default:
            $key = getPrefix($key);
    }

    $type = $redis->type($key);

    switch ($type) {
        case \Redis::REDIS_NOT_FOUND:
            return $default;

        case \Redis::REDIS_STRING:
            return $redis->get($key);

        case \Redis::REDIS_LIST:
            return $redis->lRange($key, 0, -1);

        case \Redis::REDIS_HASH:
            if ($hkey != false) {
                return $redis->hGet($key, $hkey);
            }
            else {
                return $redis->hGetAll($key);
            }
    }

    return $default;
}

//
// fetch/set file contents from redis
//

function getFiles($files)
{
    $key = md5(implode(':', $files));
    $data = get("c/$key");

    if (!$data) {
        return false;
    }

    //
    // check latest modified time
    //

    $latest = 0;
    foreach ($files as $f) {
        $mtime = filemtime($f);
        if ($mtime > $latest) {
            $latest = $mtime;
        }
    }

    //
    // check cached data isnt stale
    //

    if ($data['ts'] < $latest) {
        return false;
    }

    return $data['c'];
}

function setFiles($files, $data)
{
    $key = md5(implode(':', $files));
    set("c/$key", ['ts' => time(), 'c' => $data]);
}

//
// return all rank info
//

function getRanks($webcamsId, $normalised=0, $months=0)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	if ($months > 0) {
		$keys = array();
		$yymm = \Avos\Utils\yearsMonthsKeys($months);

		foreach ($yymm as $date) {
			$keys[] = "$webcamsId/r/$date";
		}
	}

	sort($keys);
	$ranks = array();

    foreach ($keys as $k) {
        if ($redis->type($k) != \Redis::REDIS_NOT_FOUND) {
            $tmp = $redis->lRange($k, 0, -1);
        }
        else {
            $tmp = [];
        }

		if (count($tmp) == 0) {
			$tmp = \Avos\SQLite\get($k);
		}

		if ($tmp !== false) {
			$ranks = array_merge($ranks, $tmp);
		}
	}

	$max = array('r' => 0, 'g' => 0, 'v' => 0, 'f' => 0);
	$tmax = 0;

	// find max for each attribute and of all
	foreach ($ranks as $r) {
		foreach (array('r', 'g', 'v', 'f') as $f) {
			if (!isset($r[$f])) {
				$r[$f] = 0;
			}

			if ($r[$f] > $max[$f]) {
				$max[$f] = $r[$f];
			}
			if ($r[$f] > $tmax) {
				$tmax = $r[$f];
			}
		}
	}

	// build return lists
	$ret = array(
				'epochTime' => array(),
				'dateStr' => array(),
				'rank' => array(), 
				'genderRank' => array(), 
				'viewers' => array(), 
				'followers' => array()
			);

	$map = array('r' => 'rank', 'g' => 'genderRank', 'v' => 'viewers', 'f' => 'followers');

	// normalise the numbers from 1 thru 100000
	foreach ($ranks as $r) {
		foreach (array('r', 'g', 'v', 'f') as $f) {

			// should fix old data where we didnt collect viewers+followers
			if (!isset($r[$f])) {
				$r[$f] = 0;
			}

			// in case we don't have followers
			if ($max[$f] == 0) {
				$max[$f] = 1;
			}

			if ($normalised > 0) {
				$ret[$map[$f]][] = $r[$f] * $normalised / $max[$f];
			}
			else {
				$ret[$map[$f]][] = $r[$f];
			}
		}

		$ret['epochTime'][] = $r['e'] * 60 + EPOCHSTART;
		$ret['dateStr'][] = date('y M y g:i a', $r['e'] * 60 + EPOCHSTART);
	}

	// reverse
	foreach ($ret as $k => $v) {
		$ret[$k] = array_reverse($v);
	}

	return $ret;
}

//
// get a model attribute
// * if isList then append $k to the key and use a list
//   else set a value
//

function getModel($webcamsId, $k='', $isList=false)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

    $key = "$webcamsId/p";

    if ($redis->type($key) == \Redis::REDIS_NOT_FOUND) {
        return [];
    }

	if ($isList) {
		$key .= "/$k";
		$ret = $redis->lRange($key, 0, -1);
	}
	elseif (empty($k)) {
		$ret = $redis->hGetAll($key);
	}
	else {
		$ret = $redis->hGet($key, $k);
    }

    //
    // add in any local site data
    //

    if (is_array($ret)) {
        $local = get(":m/$webcamsId");

        if ($local != false) {
            foreach ($local as $k => $v) {
                switch ($k) {
                case 'r':
                    $ret['localRankMax'] = $v;
                    break;
                case 'ts':
                    $ret['localRankMaxOn'] = $v;
                    break;
                default:
                    $ret[$k] = $v;
                }
            }
        }
    }

	return $ret;
}

//
// set a model attribute
//

function setModel($webcamsId, $k, $v, $isList=false)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	$key = "$webcamsId/p";

	try {
		if ($isList) {
			$key = "$webcamsId/$k";
			#echo "appending [$key] with [$v]\n";
			$redis->rPush($key, $v);
		}
		else {
			#echo "setting [$key.$k] to [$v]\n";
			$redis->hSet($key, $k, $v);
		}
	}
	catch (Exception $e) {
		echo "Avos\Redis\setModel unable to save data\n";
	}
}

//
// either call with just webcamsId or webcamsId + year + month
//
// redis stores elements as e, s, t
// return as epochTime, status, statusTime
//

function getHistoryMonth($webcamsId, $year='', $month='')
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	// need leading zero
	$month = sprintf("%02d", $month);
	$key = "$webcamsId/h/$year$month";

    if ($redis->type($key) != \Redis::REDIS_NOT_FOUND) {
        $tmp = $redis->lRange($key, 0, -1);
    }
    else {
        $tmp = [];
    }

	if (count($tmp) == 0) {
		$tmp = \Avos\SQLite\get($key);
	}

	$ret = array();

	foreach ($tmp as $v) {
		$ret[] = array(
					'webcamsId' => $webcamsId,
					'epochTime' => $v['e']*60 + EPOCHSTART, 
					'status' => $v['s'], 
					'statusTime' => $v['t']
				);
	}

	return $ret;
}

//
// if count=0 then just return the all else return at least count amounts
// if reverse=true then return from newest to oldest
//

function getHistory($webcamsId, $count=0, $reverse=false, $months=2)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	$keys = array();
	$ret = array();

	$year = date('y');
	$month = date('n');

	while ($months-- > 0) {
		$keys[] = "$webcamsId/h/$year" . sprintf("%02d", $month - $months);
	}

	//
	// sort the keys
	//

	if ($reverse == true) {
		rsort($keys);
	}
	else {
		sort($keys);
	}

	//
	// extract data for specified months
	//

	foreach ($keys as $key) {

        // check sqlite if the key doesnt exist in redis
        if ($redis->type($key) != \Redis::REDIS_NOT_FOUND) {
            $tmp = $redis->lRange($key, 0, -1);
        }
        else {
            $tmp = [];
        }

		if (count($tmp) == 0) {
			$tmp = \Avos\SQLite\get($key);
		}

		$tmp2 = array();

		if (is_array($tmp)) {
			foreach ($tmp as $v) {
				$tmp2[] = array(
						'webcamsId' => $webcamsId,
						'epochTime' => $v['e'], 
						'status' => $v['s'], 
						'statusTime' => $v['t']
					);
			}

			// if we need in reverse order then 
			$tmp2 = array_reverse($tmp2);

			// append to return array
			foreach ($tmp2 as $t) {
				$ret[] = $t;
			}

			// check to see if we have enough
			if ($count > 0 && count($ret) >= $count) {
				return $ret;
			}
		}
	}

	// return all we have
	return $ret;
}

//
// expand keys from short internal format
//

function expandStat($webcamsId, $year, $month, $day, $v)
{
	return array(
		'webcamsId' => $webcamsId,
		'year' => $year,
		'month' => $month,
		'day' => $day,

		'awaySessions' => $v['as'], 
		'awayTime' => $v['at'], 
		'awayTimeMax' => $v['atm'],

		'freechatSessions' => $v['fs'], 
		'freechatTime' => $v['ft'], 
		'freechatTimeMax' => $v['ftm'],

		'groupSessions' => $v['gs'], 
		'groupTime' => $v['gt'], 
		'groupTimeMax' => $v['gtm'],

		'privateSessions' => $v['ps'], 
		'privateTime' => $v['pt'], 
		'privateTimeMax' => $v['ptm']
	);
}

//
// set a daily or monthly stats entry
//

function setStat($webcamsId, $mode, $year, $month, $day, $value)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	$year = sprintf("%02d", $year);
	$month = sprintf("%02d", $month);

	if (!empty($day)) {
		$day = sprintf("%02d", $day);
	}

	if ($mode == 'm') {
		$key = "$webcamsId/m/$year$month";
	}
	else {
		$key = "$webcamsId/d/$year$month$day";
	}

	#if (DEBUG) echo "setting ($key) => "; print_r($value);
	$redis->set($key, $value);
}

//
// return only a single record based on the key info provided
//

function getStat($webcamsId, $mode="m", $year="", $month="", $day="", $expandNames=true)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	// year
	if (!empty($year)) {
		$year = sprintf("%02d", (int)$year);
	}

	// month
	if (!empty($month)) {
		$month = sprintf("%02d", (int)$month);
	}

	// day
	if (!empty($day)) {
		$day = sprintf("%02d", (int)$day);
	}

	// create the key
	$key = "$webcamsId/$mode/$year$month";
	if ($mode == 'd') {
		$key .= $day;
	}

    // expand if we need to

    if ($redis->type($key) == \Redis::REDIS_NOT_FOUND) {
		$ret = \Avos\SQLite\get($key);
    }
    else {
        $ret = $redis->get($key);
    }

	if ($expandNames == true) {
		$ret = expandStat($webcamsId, $year, $month, $day, $ret);
	}

	return $ret;
}

//
// mode: m=monthly, d=daily
//
// XXXX/m/YYMM     => single entry
// XXXX/m/YY       => multiple entries
// XXXX/d/YYMMDD   => single entry
// XXXX/d/YYMM     => multiple entries
// XXXX/d/YY       => multiple entries
//

function getStats($webcamsId, $mode="m", $year="*", $month="*", $day="*", $reverse=false, $expandNames=true)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	// year
	if (!empty($year) && $year != '*') {
		$year = sprintf("%02d", $year);
	}

	// month
	if (!empty($month) && $month != '*') {
		$month = sprintf("%02d", $month);
	}

	// day
	if ($mode == 'd' && !empty($day) && $day != '*') {
		$day = sprintf("%02d", $day);
	}

	// create key with 0 or 1 wildcard '*'
	$search = "$webcamsId/$mode/$year";

	if ($year != "*") {
		$search .= $month;

		if ($month != "*" && $mode == 'd') {
			$search .= $day;
		}
	}

	//
	// search for specified years/months
	//

	$keys = array();

	if ((int)$month >= 0) {
		if ($mode == 'd') {
			for ($i=1; $i<31; $i++) {
				$keys[] = sprintf("$webcamsId/d/%s%s%02d", $year, $month, $i);
			}
		}
		else {
			//$keys = $redis->keys($search);
			echo "Avos\Redis\getModel() will not run search\n";
			return array();
		}
	}

	//
	// search back required number of months
	// e.g: $month = -10 then search back 10 months
	//

	else {
		$days = $mode == 'd' ? true : false;
		$dates = \Avos\Utils\yearsMonthsKeys(0 - $month, $days);

		foreach ($dates as $d) {
			$keys[] = sprintf("$webcamsId/$mode/$d");
		}
	}

	//
	// sort the keys
	//

	#echo "search=$search\n";exit;
	$ret = array();

	if ($reverse == true) {
		rsort($keys);
	}
	else {
		sort($keys);
	}

	//
	// extract the stats
	//

	foreach ($keys as $key) {

        if ($redis->get($key) != \Redis::REDIS_NOT_FOUND) {
            $v = $redis->get($key);
        }

		if (empty($v) || count($v) == 0) {
			$v = \Avos\SQLite\get($key);
		}

		if (is_array($v) && count($v) > 0) {
			preg_match("!/(\d\d)(\d\d)(\d\d)?$!", $key, $c);
			$year = (int)$c[1];
			$month = (int)$c[2];
			$day = isset($c[3]) ? (int)$c[3] : '';

			if ($expandNames == true) {
				$ret[] = expandStat($webcamsId, $year, $month, $day, $v);
			}
			else {
				$ret[] = $v;
			}
		}
	}

	return $ret;
}

//
// to fetch all stats we first need to search sqlite as its very fast to search (unlike redis)
// then fetch data from then until current month from redis
//

function getStatsSummary($webcamsId, $flat=false)
{
    $redis = handle();
    $ck = "$webcamsId/c";

    $ct = getConfig('webcams', 'statsCacheExpiryHours', 8) * 3600; # 8 hours

    if ($redis->type($ck) !== \Redis::REDIS_NOT_FOUND) {
        $redis->expire($ck, $ct);

        if ($flat) {
            return getStatsSummaryFlat($webcamsId, $redis->hGetAll($ck));
        }
        else {
            return $redis->hGetAll($ck);
        }
    }
    else {
        $stats = getAllStats($webcamsId);
    }

    //////////////////////////////////////////////////////////////////////
    // create a status summary
    //
    // last 7 days, month, all time
    // - max viewers
    // - average viewers
    // - new followers
    // - time online
    // - daily time online
    // - days online
    // - longest inactivity date range
    // - average inactivity
    //////////////////////////////////////////////////////////////////////

    //
    // skelton return hash
    //

    $elt = [ 
        'viewersTotal' => 0, 
        'viewersMax' => 0, 
        'viewersAvg' => 0, 

        'followersNew' => 0, 
        'followersMin' => 0, 
        'followersMax' => 0, 

        'rankTotal' => 0,
        'rankMax' => -1,
        'rankAvg' => -1,

        'onlineTime' => 0, 
        'onlineDays' => 0,
        'onlineDaily' => 0,

        'freechatTime' => 0,
        'freechatLongest' => 0,
        'freechatSessions' => 0,
        'freechatAvg' => 0,

        'groupTime' => 0,
        'groupLongest' => 0,
        'groupSessions' => 0,
        'groupAvg' => 0,

        'privateTime' => 0,
        'privateLongest' => 0,
        'privateSessions' => 0,
        'privateAvg' => 0,

        'popularRank' => 0,
        'trendingRank' => 0
    ];

    $s = [ 'd' => $elt, 'm' => $elt, 'a' => $elt ];

    //
    // time intervals for return hash
    // set 'now' as the last time model online
    //

    foreach ($stats['r'] as $months => $month) {
        foreach ($month as $r) {
            if (isset($r['e'])) {
                $now = $r['e'] * 60 + EPOCHSTART;
            }
        }
    }

    $day = date('ymd');
    $now = time();
    $now24h = $now - 86400;
    $now30d = $now - 30 * 86400;

    //
    // start by iterating around the ranks/followers/viewers logs
    //

    foreach ($stats['r'] as $months => $month) {
        foreach ($month as $r) {

            if (!isset($r['e'])) {
                continue;
            }

            $epoch = $r['e'] * 60 + EPOCHSTART;
            $today = date('ymd', $epoch);


            ////////////////////////////////////////////////
            // not all sites have all data

            foreach (['v', 'r', 'f'] as $f) {
                if (!isset($r[$f])) {
                    $r[$f] = 0;
                }
            }

            ////////////////////////////////////////////////
            // viewers all time
            //

            #$s['a']['onlineDays']++;
            if (isset($r['v'])) {
                $s['a']['viewersTotal'] += $r['v'];

                if ($r['v'] > $s['a']['viewersMax']) {
                    $s['a']['viewersMax'] = $r['v'];
                }
            }

            //
            // viewers last 30 days
            //

            if ($epoch >= $now30d) {
                #$s['m']['onlineDays']++;
                $s['m']['viewersTotal'] += $r['v'];

                if ($r['v'] > $s['m']['viewersMax']) {
                    $s['m']['viewersMax'] = $r['v'];
                }
            }

            //
            // viewers last 24 hrs
            //

            if ($epoch >= $now24h) {
                #$s['d']['onlineDays']++;
                $s['d']['viewersTotal'] += $r['v'];

                if ($r['v'] > $s['d']['viewersMax']) {
                    $s['d']['viewersMax'] = $r['v'];
                }
            }

            ////////////////////////////////////////////////
            // rank all time
            //

            $s['a']['rankTotal'] += $r['r'];

            if ($r['r'] < $s['a']['rankMax'] || $s['a']['rankMax'] == -1) {
                $s['a']['rankMax'] = $r['r'];
            }

            //
            // rank last 30 days
            //

            if ($epoch >= $now30d) {
                $s['m']['rankTotal'] += $r['r'];

                if ($r['r'] < $s['m']['rankMax'] || $s['m']['rankMax'] == -1) {
                    $s['m']['rankMax'] = $r['r'];
                }
            }

            //
            // rank last 24 hrs
            //

            if ($epoch >= $now24h) {
                $s['d']['rankTotal'] += $r['r'];

                if ($r['r'] < $s['d']['rankMax'] || $s['d']['rankMax'] == -1) {
                    $s['d']['rankMax'] = $r['r'];
                }
            }

            ////////////////////////////////////////////////
            // followers all time
            //

            if (!isset($r['f'])) {
                $r['f'] = 0;
            }

            $s['a']['followersMax'] = $r['f'];

            if ($s['a']['followersMin'] == 0) {
                $s['a']['followersMin'] = $r['f'];
            }

            //
            // followers last 30 days
            //

            if ($epoch >= $now30d) {
                $s['m']['followersMax'] = $r['f'];

                if ($s['m']['followersMin'] == 0) {
                    $s['m']['followersMin'] = $r['f'];
                }
            }

            //
            // followers last 24 hours
            //

            if ($epoch >= $now24h) {
                $s['d']['followersMax'] = $r['f'];

                if ($s['d']['followersMin'] == 0) {
                    $s['d']['followersMin'] = $r['f'];
                }
            }

            ////////////////////////////////////////////////
            // days online

            if ($today != $day) {
                $s['a']['onlineDays']++;

                if ($epoch >= $now30d) {
                    $s['m']['onlineDays']++;
                }

                if ($epoch >= $now24h) {
                    $s['d']['onlineDays'] = 1;
                }

                $day = $today;
            }
        }
    }

    foreach (['d', 'm', 'd'] as $i) {
        if ($s[$i]['onlineDays'] > 0) {
            foreach (['viewers', 'rank'] as $t) {
                $s[$i][$t . 'Avg'] = (int)($s[$i][$t . 'Total'] / $s[$i]['onlineDays']);
            }
        }
    }

    #$s['d']['viewersAvg'] = (int)($s['d']['viewersTotal'] / $s['d']['onlineDays']);
    #$s['m']['viewersAvg'] = (int)($s['m']['viewersTotal'] / $s['m']['onlineDays']);
    #$s['a']['viewersAvg'] = (int)($s['a']['viewersTotal'] / $s['a']['onlineDays']);

    #$s['d']['rankAvg'] = (int)($s['d']['rankTotal'] / $s['d']['onlineDays']);
    #$s['m']['rankAvg'] = (int)($s['m']['rankTotal'] / $s['m']['onlineDays']);
    #$s['a']['rankAvg'] = (int)($s['a']['rankTotal'] / $s['a']['onlineDays']);

    $s['d']['followersNew'] = $s['d']['followersMax'] - $s['d']['followersMin'];
    $s['m']['followersNew'] = $s['m']['followersMax'] - $s['m']['followersMin'];
    $s['a']['followersNew'] = $s['a']['followersMax'] - $s['a']['followersMin'];

    //
    // loop around the daily data which contains data on freechat/group/private sessions
    //

    $typeName = [
        'f' => 'freechat',
        'g' => 'group',
        'p' => 'private'
    ];

    foreach ($stats['d'] as $day => $d) {

        $epoch = strtotime(\Avos\Utils\yymmddToDate($day));

        foreach (['f', 'g', 'p'] as $t) {

            foreach (['t', 's', 'tm'] as $f) {
                if (!isset($d[$t.$f])) {
                    $d[$t.$f] = 0;
                }
            }

            $name = $typeName[$t];

            //
            // all time
            //

            $s['a']['onlineTime'] += $d[$t . 't'];
            $s['a'][$name . 'Time'] += $d[$t . 't'];
            $s['a'][$name . 'Sessions'] += $d[$t . 's'];

            if ($d[$t . 'tm'] > $s['a'][$name . 'Longest']) {
                $s['a'][$name . 'Longest'] += $d[$t . 'tm'];
            }

            //
            // last 30 days
            //

            if ($epoch >= $now30d) {
                $s['m']['onlineTime'] += $d[$t . 't'];
                $s['m'][$name . 'Time'] += $d[$t . 't'];
                $s['m'][$name . 'Sessions'] += $d[$t . 's'];

                if ($d[$t . 'tm'] > $s['m'][$name . 'Longest']) {
                    $s['m'][$name . 'Longest'] += $d[$t . 'tm'];
                }
            }

            //
            // last 24 hours
            //

            if ($epoch >= $now24h) {
                $s['d']['onlineTime'] += $d[$t . 't'];
                $s['d'][$name . 'Time'] += $d[$t . 't'];
                $s['d'][$name . 'Sessions'] += $d[$t . 's'];

                if ($d[$t . 'tm'] > $s['d'][$name . 'Longest']) {
                    $s['d'][$name . 'Longest'] += $d[$t . 'tm'];
                }
            }
        }
    }

    foreach (['f', 'g', 'p'] as $t) {
        $name = $typeName[$t];
        foreach (['a', 'm', 'd'] as $p) {
            if ($s[$p][$name . 'Sessions'] > 0) {
                $s[$p][$name . 'Avg'] = (int)($s[$p][$name . 'Time'] / $s[$p][$name . 'Sessions']);
            }
            else {
                $s[$p][$name . 'Avg'] = 0;
            }
        }
    }

    foreach (['a', 'm', 'd'] as $p) {
        if ($s[$p]['onlineDays'] > 0) {
        $s[$p]['onlineDaily'] = (int)($s[$p]['onlineTime'] / $s[$p]['onlineDays']);
            unset($s[$p]['rankTotal']);
            }
    }

    //
    // cache the redis summary from redis in redis (!)
    //

    $redis->hMSet($ck, $s);
    $redis->expire($ck, $ct);

    if ($flat) {
        return getStatsSummaryFlat($webcamsId, $redis->hGetAll($ck));
    }
    else {
        return $redis->hGetAll($ck);
    }

    return $s;
}

function getStatsSummaryFlat($webcamsId, $stats)
{
    $model = getWebcam([ 'id' => $webcamsId ]);

    if (count($model) == 0) {
        $model = getWebcam([ 'id' => $webcamsId ], [], '', true);
    }

    $model = count($model) == 1 ? $model[0] : [];

    //
    // if offline reset some values
    //

    if ($model['status'] == 'Offline') {
        $model['rank'] = 0;
        $model['genderRank'] = 0;
        $model['viewers'] = 0;
    }

    //
    // create some extra values
    //

    $now = time();

    /*
    foreach (['addedOn', 'updatedOn'] as $f) {
        if (isset($model[$f])) {
            $model[$f . 'Ago'] = $now - $model[$f];
            $model[$f . 'AgoDays'] = sprintf('%.1f', $model[$f . 'Ago'] / 86400);
            $model[$f . 'AgoMonths'] = sprintf('%.1f', $model[$f . 'AgoDays'] / 30);
            $model[$f . 'AgoYears'] = sprintf('%.1f', $model[$f . 'AgoDays'] / 365);
        }
    }
    */

    //
    // if online add in statusTime
    //

    $t = \Avos\Database\queryAssoc('SELECT epochTime FROM history WHERE webcamsId=:id ORDER BY epochTime DESC LIMIT 1', ['id' => $webcamsId]);

    if (count($t) == 1) {
        $model['statusTime'] = $now - $t[0]['epochTime'];
        //$model['statusTimeMins'] = sprintf('%.1f', $model['statusTime'] / 60);
        //$model['statusTimeHours'] = sprintf('%.1f', $model['statusTimeMins'] / 60);
    }

    //
    // add in stats summary
    //

    $skip = ['popularRank' => 1, 'trendingRank' => 1, 'rankMax' => 1];

    foreach ($stats['a'] as $k => $v) {
        if (!isset($skip[$k])) {
            $model[$k] = $v;
        }
    }

    foreach ($stats['d'] as $k => $v) {
        if (!isset($skip[$k])) {
            $model[$k . 'Day'] = $v;
        }
    }

    foreach ($stats['m'] as $k => $v) {
        if (!isset($skip[$k])) {
            $model[$k . 'Month'] = $v;
        }
    }

    return $model;
}

function getAllStats($webcamsId)
{
    //
    // search and fetch all data from sqlite
    //

    $redis = handle();
    $t = \Avos\SQLite\search("$webcamsId/*");
    $s = [ 'd' => [], 'h' => [], 'r' => [] ];

    foreach ($t as $k) {
        if (preg_match('!([dhr])\/(\d+)!', $k, $c)) {
            $s[$c[1]][$c[2]] = \Avos\SQLite\get("$webcamsId/$c[1]/$c[2]");
        }
    }

    //
    // fetch daily stats from redis from the final date till today
    //

    $today = date('ymd');
    $last = array_key_last($s['d']);

    foreach (\Avos\Utils\yearsMonthsDaysKeys($last, $today) as $d) {
        if ($redis->type("$webcamsId/d/$d") != \Redis::REDIS_NOT_FOUND) {
            $t = $redis->get("$webcamsId/d/$d");
        }

        if (!empty($t)) {
            $s['d'][$d] = $t;
        }
    }

    //
    // status timeline
    //

    $today = date('ym' . '01');

    foreach (['h', 'r'] as $k) {

        $last = array_key_last($s[$k]) . '01';

        foreach (\Avos\Utils\yearsMonthsDaysKeys($last, $today, true) as $m) {
            if ($redis->type("$webcamsId/$k/$m") != \Redis::REDIS_NOT_FOUND) {
                $t = $redis->lRange("$webcamsId/$k/$m", 0, -1);
            }

            if (!empty($t)) {
                $s[$k][$m] = $t;
            }
        }
    }

    return $s;
}










